@@ -648,8 +648,12 @@ btree_WriteB (btree, fn)
   /* Write out the header information */
   fwrite (&type, sizeof (FOURB), 1, fn);
 
-  /* Number of nodes */
-  num_nodes = btree_GetNumNodes (btree);
+  /*
+   * We must walk the tree first, to figure out
+   * the numberof valid nodes.
+   */
+
+  num_nodes = btree_GetRealNumNodes (btree);
   fwrite (&num_nodes, sizeof (FOURB), 1, fn);
 
   /* Tree Name */
