@@ -110,13 +110,13 @@ Members (return_ptr, n_args, d_arg)
      int n_args;
      Datum *d_arg;
 {
-  ListNode *List = 0;
+  ListNode *L = 0;
   Matrix *m = 0;
 
   if (n_args == 1)
   {
-    List = bltin_get_list ("members", d_arg, 1);
-    m = (Matrix *) btree_members ((e_data (List)));
+    L = bltin_get_list ("members", d_arg, 1);
+    m = (Matrix *) btree_members ((e_data (L)));
   }
   else
   {
@@ -125,7 +125,7 @@ Members (return_ptr, n_args, d_arg)
   }
 
   *return_ptr = (VPTR) m;
-  remove_tmp_destroy (List);
+  remove_tmp_destroy (L);
 
 }
 
@@ -1527,6 +1527,11 @@ bltin_get_numeric_matrix (bltin_name, arg_array, arg_number)
   return (ent);
 }
 
+/*
+ * Get a builtin argument. Coerce it into a matrix if
+ * possible.
+ */
+
 ListNode *
 bltin_get_matrix (bltin_name, arg_array, arg_number)
      char *bltin_name;
