@@ -5,11 +5,6 @@
    op-code functions. Some of the class related op-code functions
    are off in files like op.c, op_matrix.c, etc. */
 
-/* At this point in time code.c is a working implementation. It is
-   NOT a fast implementation. There are many things that can, and
-   will be done to make the interpreter faster once the language is
-   finished and tested. 8/4/92 */
-
 /*  This file is a part of RLaB ("Our"-LaB)
    Copyright (C) 1992, 1993, 1994  Ian R. Searle
 
@@ -38,6 +33,9 @@
 #include "mem.h"
 #include "y.tab.h"
 #include "bltin.h"
+#include "matrix.h"
+#include "matop1.h"
+#include "matop2.h"
 #include "function.h"
 #include "r_string.h"
 #include "util.h"
@@ -67,6 +65,22 @@ typedef struct _frame Frame;
 static Frame *frame;		/* Frame stack */
 static Frame *fp;		/* Frame pointer */
 
+struct _forloop
+{
+  int cnt;			/* Current location in v. */
+  int n;			/* Size of v. */
+  int type;			/* Numeric or String. */
+  int jmp;			/* The jump distance. */
+  ListNode *var;		/* The for-loop variable. */
+  Matrix *v;			/* The for-loop vector. */
+};
+
+typedef struct _forloop ForLoop;
+#define NUM_REAL         0
+#define NUM_COMPLEX      1
+
+static List *flstack;		/* For-Loop stack */
+
 static int NSTACK = 1024;	/* Fixed stack size */
 static Datum *stack;		/* the stack */
 static Datum *stackp;		/* next free spot on stack */
@@ -75,10 +89,6 @@ static Program *program;	/* The current program for code generation */
 static Inst *pc;		/* Program counter during execution */
 static int progoff;		/* The current program offset, from beginning */
 
-static int returning;		/* 1 if return stmt seen */
-static int breaking = 0;
-static int continuing = 0;
-
 /* dissassem.c */
 extern void diss_assemble _PROTO ((Inst * p, int progoff));
 
@@ -99,9 +109,7 @@ void print _PROTO ((Datum d));
 void inc _PROTO ((void));
 void dec _PROTO ((void));
 Datum assign _PROTO ((Datum d1, Datum d2));
-void whilecode _PROTO ((void));
-void forcode _PROTO ((void));
-void ifcode _PROTO ((void));
+Datum olist_assign _PROTO ((Datum d1, Datum d2));
 void bltin _PROTO ((ListNode * sp, int narg, int popf));
 void function_call _PROTO ((void));
 void function_call_1 _PROTO ((void));
@@ -290,6 +298,12 @@ init_stack ()
   stackp = stack;
 }
 
+void
+init_flstack ()
+{
+  flstack = list_Create ();
+}
+
 /* **************************************************************
  * Functions that create and manipulate program arrays.
  * ************************************************************** */
@@ -436,6 +450,24 @@ set_stackp (new_stackp)
   stackp = new_stackp;
 }
 
+/* **************************************************************
+ * Handle clean up operations for the for-loop stack.
+ * ************************************************************** */
+
+void
+clean_up_bc_lists ()
+{
+  list_DestroyAllNodes (flstack);
+}
+
+void
+forloop_Destroy (fl)
+     ForLoop *fl;
+{
+  matrix_Destroy (fl->v);
+  FREE (fl);
+}
+
 /* **************************************************************
  * Initialize the program for code generation.
  * ************************************************************** */
@@ -448,11 +480,6 @@ initcode ()
   /* Reset parser/program offset */
   progoff = 0;
 
-  /* Reset flow control indicators */
-  returning = 0;
-  breaking = 0;
-  continuing = 0;
-
   if (line_nos)
   {
     /* Set the file name for the next section of code */
@@ -605,7 +632,7 @@ function_call ()
   nargs = (*pc).op_code;
 
   /* Entity holding user/builtin function */
-  
+
   fst = pop ();
   sp = fst.u.ent;
 
@@ -756,25 +783,25 @@ userf (sp, nargs, self)
    * entities so that they aren't cleaned from the list.
    * Also change CONSTANTs into scalars.
    */
-  
+
   fargs = function_GetArgPtr (e_data (sp));
   for (i = 1; i <= fp->nargs; i++)
   {
-    ListNode *lfarg = list_GetNodeByPos (fargs, fp->nargs+1-i);
+    ListNode *lfarg = list_GetNodeByPos (fargs, fp->nargs + 1 - i);
     char *name = lvar_GetName (e_data (lfarg));
-      
+
     if (fp->argn[i - fp->nargs].type == ENTITY
 	&& fp->argn[i - fp->nargs].u.ent->key == 0)
     {
       fp->argn[i - fp->nargs].u.ent->key = cpstr (tag);
       if (fp->argn[i - fp->nargs].u.ent->type == MATRIX)
-	matrix_SetName (e_data (fp->argn[i - fp->nargs].u.ent), cpstr(name));
+	matrix_SetName (e_data (fp->argn[i - fp->nargs].u.ent), cpstr (name));
       else if (fp->argn[i - fp->nargs].u.ent->type == SCALAR)
-	scalar_SetName (e_data (fp->argn[i - fp->nargs].u.ent), cpstr(name));
+	scalar_SetName (e_data (fp->argn[i - fp->nargs].u.ent), cpstr (name));
       else if (fp->argn[i - fp->nargs].u.ent->type == STRING)
-	string_SetName (e_data (fp->argn[i - fp->nargs].u.ent), cpstr(name));
+	string_SetName (e_data (fp->argn[i - fp->nargs].u.ent), cpstr (name));
       else if (fp->argn[i - fp->nargs].u.ent->type == BTREE)
-	btree_SetName (e_data (fp->argn[i - fp->nargs].u.ent), cpstr(name));
+	btree_SetName (e_data (fp->argn[i - fp->nargs].u.ent), cpstr (name));
       else if (fp->argn[i - fp->nargs].u.ent->type == UNDEF)
 	;
       else
@@ -787,7 +814,7 @@ userf (sp, nargs, self)
 		     scalar_Destroy);
       fp->argn[i - fp->nargs].u.ent->key = cpstr (tag);
       fp->argn[i - fp->nargs].type = ENTITY;
-      scalar_SetName (e_data (fp->argn[i - fp->nargs].u.ent), cpstr(name));
+      scalar_SetName (e_data (fp->argn[i - fp->nargs].u.ent), cpstr (name));
     }
     else if (fp->argn[i - fp->nargs].type == iCONSTANT)
     {
@@ -796,7 +823,7 @@ userf (sp, nargs, self)
 		     scalar_Destroy);
       fp->argn[i - fp->nargs].u.ent->key = cpstr (tag);
       fp->argn[i - fp->nargs].type = ENTITY;
-      scalar_SetName (e_data (fp->argn[i - fp->nargs].u.ent), cpstr(name));
+      scalar_SetName (e_data (fp->argn[i - fp->nargs].u.ent), cpstr (name));
     }
   }
 
@@ -821,8 +848,9 @@ userf (sp, nargs, self)
   fp->self = self;
   /* Now execute the user-function */
   execute ((Inst *) function_GetCodePtr (listNode_GetData (sp)));
-  returning = 0;
 
+  pc = fp->retpc;
+  --fp;
 }
 
 /*
@@ -888,7 +916,7 @@ load_local_var (f, nargs)
     if ((cnode = list_GetNodeByKey (function_GetArgPtr (f), name)))
     {
       /* COPY */
-      copy = copy_arg (cnode, name);
+      copy = copy_arg (cnode, cpstr (name));
 
       if (copy)			/* handle UNDEFs */
 	d.u.ent = install_tmp_switch ((int) *((int *) copy), copy);
@@ -903,7 +931,7 @@ load_local_var (f, nargs)
       if (!(strcmp ("nargs", name)))
       {
 	/* Set local variable nargin (number of arguments). */
-	d.u.ent = install_tmp (SCALAR, 
+	d.u.ent = install_tmp (SCALAR,
 			       scalar_Create ((double) nargs),
 			       scalar_Destroy);
       }
@@ -1055,14 +1083,10 @@ ret_from_func (rd)
       }
     }
   }
-  
+
   if (fp->self)
-    pop ();
+    extern_pop ();
 
-  pc = fp->retpc;
-
-  --fp;
-  returning = 1;
   return (rd);
 }
 
@@ -1103,17 +1127,13 @@ function_default_return ()
   }
 
   if (fp->self)
-    pop ();
-
-  pc = fp->retpc;
-
-  --fp;
-  returning = 1;
+    extern_pop ();
 
   /* 
    * Create a 0 scalar and push it on the stack so print()
    * will have something to do.
    */
+
   d.u.ent = install_tmp (SCALAR, scalar_Create (0.0), scalar_Destroy);
   d.type = ENTITY;
   push (d);
@@ -1147,7 +1167,7 @@ execute (p)
 
   pc = p;
 
-  while (!returning)
+  while (1)
   {
     if (interrupt)
     {
@@ -1159,62 +1179,62 @@ execute (p)
     switch ((*pc++).op_code)
     {
     case STOP:
-      pc--;
+      /* pc--; */
       return;
       break;
 
     case OP_PUSH_VAR:
       new.u.ent = (ListNode *) ((*pc++).ptr);
       new.type = ENTITY;
-      push (new);
+      *stackp++ = new;
       break;
 
     case OP_PUSH_ARG:
       offset = (*pc++).op_code;
       new = fp->argn[offset - fp->nargs];
-      push (new);
+      *stackp++ = new;
       break;
 
     case OP_PUSH_LOCAL_VAR:
       offset = (*pc++).op_code;
       new = fp->localn[offset - fp->n_local];
       new.type = ENTITY;
-      push (new);
+      *stackp++ = new;
       break;
 
     case OP_ADD:
-      d2 = pop ();
-      d1 = pop ();
+      d2 = *--stackp;
+      d1 = *--stackp;;
       new = addition_op (d1, d2);
-      push (new);
+      *stackp++ = new;
       break;
 
     case OP_SUB:
-      d2 = pop ();
-      d1 = pop ();
+      d2 = *--stackp;
+      d1 = *--stackp;;
       new = subtraction_op (d1, d2);
-      push (new);
+      *stackp++ = new;
       break;
 
     case OP_MUL:
-      d2 = pop ();
-      d1 = pop ();
+      d2 = *--stackp;
+      d1 = *--stackp;;
       new = multiply_op (d1, d2);
-      push (new);
+      *stackp++ = new;
       break;
 
     case OP_DIV:
-      d2 = pop ();
-      d1 = pop ();
+      d2 = *--stackp;
+      d1 = *--stackp;;
       new = rdivide (d1, d2);
-      push (new);
+      *stackp++ = new;
       break;
 
     case OP_LDIV:
-      d2 = pop ();
-      d1 = pop ();
+      d2 = *--stackp;
+      d1 = *--stackp;;
       new = ldivide (d1, d2);
-      push (new);
+      *stackp++ = new;
       break;
 
     case OP_NEGATE:
@@ -1224,35 +1244,229 @@ execute (p)
       break;
 
     case OP_POWER:
-      d2 = pop ();
-      d1 = pop ();
+      d2 = *--stackp;
+      d1 = *--stackp;;
       new = power_op (d1, d2);
-      push (new);
+      *stackp++ = new;
       break;
 
     case OP_ASSIGN:
-      d1 = pop ();
-      d2 = pop ();
+      d1 = *--stackp;
+      d2 = *--stackp;;
       new = assign (d2, d1);
-      push (new);
+      *stackp++ = new;
       break;
 
-    case OP_FOR:
-      forcode ();
+    case OP_FOR_LOOP_I:
+      {
+	Datum vec, var;
+	ListNode *lnode;
+	ForLoop *fl = (ForLoop *) MALLOC (sizeof (ForLoop));
+
+	/* Get the vector and the var from the stack */
+	vec = pop ();
+	var = pop ();
+
+	/* Get the jump distance from the program. */
+	fl->jmp = (int) (*pc).op_code;
+
+	/* Keep track of the for-loop variable. */
+	fl->var = (ListNode *) var.u.ent;
+
+	/* Copy the vector into flstack */
+	switch (vec.type)
+	{
+	case CONSTANT:
+	  fl->v = matrix_Create (1, 1);
+	  MAT (fl->v, 1, 1) = vec.u.val;
+	  break;
+
+	case iCONSTANT:
+	  fl->v = matrix_CreateC (1, 1);
+	  MATr (fl->v, 1, 1) = 0.0;
+	  MATi (fl->v, 1, 1) = vec.u.val;
+	  break;
+
+	case ENTITY:
+	  switch (e_type (vec.u.ent))
+	  {
+	  case SCALAR:
+	    {
+	      Scalar *s = (Scalar *) e_data (vec.u.ent);
+	      if (SVALi (s) == 0.0)
+	      {
+		fl->v = matrix_Create (1, 1);
+		MAT (fl->v, 1, 1) = SVALr (s);
+	      }
+	      else
+	      {
+		fl->v = matrix_CreateC (1, 1);
+		MATr (fl->v, 1, 1) = SVALr (s);
+		MATi (fl->v, 1, 1) = SVALi (s);
+	      }
+	    }
+
+	    /* Try and clean up the original vector expression */
+	    remove_tmp_destroy (vec.u.ent);
+	    break;
+
+	  case MATRIX:
+	    if (e_name (vec.u.ent) == 0)
+	    {
+	      /* Skip the copy (be fast). */
+	      fl->v = (Matrix *) remove_tmp (vec.u.ent);
+	    }
+	    else
+	    {
+	      fl->v = matrix_Copy (e_data (vec.u.ent));
+	    }
+	    break;
+
+	  case STRING:
+	    fl->v = matrix_CreateS (1, 1);
+	    MATs (fl->v, 1, 1) = cpstr (string_GetString (e_data (vec.u.ent)));
+
+	    /* Try and clean up the original vector expression */
+	    remove_tmp_destroy (vec.u.ent);
+	    break;
+
+	  default:
+	    error_1 ("Invalid vector expression for for-loop", 0);
+	    break;
+	  }
+	}
+
+	/*
+	 * Set the cnt part of the ForLoop to 0. We will use it to
+	 * mark our location in the vector as we step through.
+	 */
+
+	fl->cnt = 0;
+
+	/*
+	 * Set the size of the loop-vector.
+	 */
+
+	fl->n = MNR (fl->v) * MNC (fl->v);
+	if (fl->n != 0)
+	{
+	  /*
+	   * Check the variable, and initialize.
+	   * The parser insures that we get a variable (ENTITY).
+	   * The loop-variable will either be SCALAR (numeric)
+	   * or STRING, depending on the vector (fl->v).
+	   */
+
+	  if (MTYPE (fl->v) == REAL || MTYPE (fl->v) == COMPLEX)
+	  {
+	    /* Force var to be SCALAR (numeric) */
+	    if (e_type (var.u.ent) != SCALAR)
+	    {
+	      listNode_DestroyDataOnly (var.u.ent);
+	      listNode_AttachData (var.u.ent, SCALAR,
+				   scalar_Create (0.0),
+				   scalar_Destroy);
+	      scalar_SetName (e_data (var.u.ent),
+			      cpstr (e_name (var.u.ent)));
+	    }
+
+	    /* Initialize */
+	    if (MTYPE (fl->v) == REAL)
+	    {
+	      fl->type = NUM_REAL;
+	      SVALr (e_data (var.u.ent)) = MATrv (fl->v, fl->cnt++);
+	    }
+	    else if (MTYPE (fl->v) == COMPLEX)
+	    {
+	      fl->type = NUM_COMPLEX;
+	      SVALr (e_data (var.u.ent)) = MATcvr (fl->v, fl->cnt);
+	      SVALi (e_data (var.u.ent)) = MATcvi (fl->v, fl->cnt++);
+	    }
+	  }
+	  else
+	  {
+	    fl->type = STRING;
+	    /* Force var to be STRING */
+	    if (e_type (var.u.ent) != STRING)
+	    {
+	      listNode_DestroyDataOnly (var.u.ent);
+	      listNode_AttachData (var.u.ent, STRING,
+				   string_Create (0),
+				   string_Destroy);
+	      string_SetName (e_data (var.u.ent),
+			      cpstr (e_name (var.u.ent)));
+	    }
+
+	    /* Initialize */
+	    string_Set (e_data (var.u.ent), cpstr (MATsv (fl->v, fl->cnt++)));
+	  }
+
+	  /* Add our new loop struct to BACK of the list */
+	  lnode = listNode_AttachData (listNode_Create (), 0,
+				       fl, forloop_Destroy);
+
+	  list_PushNode (flstack, lnode);
+
+	  /* Continue loop execution. */
+	  pc++;
+	}
+	else
+	{
+	  /* Loop-vector has zero length, jump to end. */
+	  fl->type = NUM_REAL;
+	  if (e_type (var.u.ent) != MATRIX)
+	  {
+	    listNode_DestroyDataOnly (var.u.ent);
+	    listNode_AttachData (var.u.ent, MATRIX,
+				 matrix_Create (0, 0),
+				 matrix_Destroy);
+	    matrix_SetName (e_data (var.u.ent),
+			    cpstr (e_name (var.u.ent)));
+	  }
+
+	  /*
+	   * Jump to end and let OP_FOR_LOOP cleanup.
+	   * Push something on the list so that for-loop-done
+	   * doesn't create trouble.
+	   */
+
+	  lnode = listNode_AttachData (listNode_Create (), 0,
+				       fl, forloop_Destroy);
+	  list_PushNode (flstack, lnode);
+	  pc = pc + fl->jmp;
+	}
+      }
+      break;
+
+    case OP_FOR_LOOP_DONE:
+      {
+	/* Pop flstack, and cleanup. */
+	ListNode *lnode = list_PopNode (flstack);
+	listNode_Destroy (lnode);
+      }
+      break;
+
+    case OP_FOR_LOOP_DONE_RET:
+      {
+	/* Pop flstack, and cleanup. */
+	ListNode *lnode = list_PopNode (flstack);
+	listNode_Destroy (lnode);
+      }
+      return;
       break;
 
     case OP_EL_MUL:
-      d2 = pop ();
-      d1 = pop ();
+      d2 = *--stackp;
+      d1 = *--stackp;;
       new = element_multiply (d1, d2);
-      push (new);
+      *stackp++ = new;
       break;
 
     case OP_EL_DIV:
-      d2 = pop ();
-      d1 = pop ();
+      d2 = *--stackp;
+      d1 = *--stackp;;
       new = el_rdivide (d1, d2);
-      push (new);
+      *stackp++ = new;
       break;
 
     case OP_EL_LDIV:
@@ -1348,12 +1562,112 @@ execute (p)
       push (new);
       break;
 
-    case OP_IF:
-      ifcode ();
+    case OP_IFSJMP:
+      {
+	int d_val, ijmp;
+
+	/* 
+	 * Jump logic:
+	 * Pop stack. If pop'ed value is TRUE continue execution at same
+	 * point. If pop'ed value is FALSE jump to the specified value
+	 * one place behind the instruction.
+	 */
+
+	d = pop ();		/* The truth value */
+	ijmp = (int) (*pc).op_code;	/* The jump value */
+	d_val = 0;
+
+	switch (d.type)
+	{
+	case CONSTANT:
+	case iCONSTANT:
+	  d_val = d.u.val;
+	  break;
+
+	case ENTITY:
+	  switch (e_type (d.u.ent))
+	  {
+	  case SCALAR:
+	    d_val = SVALr (e_data (d.u.ent));
+	    break;
+
+	  case MATRIX:
+	    {
+	      Matrix *m;
+	      m = (Matrix *) e_data (d.u.ent);
+	      if (MNR (m) == 1 && MNC (m) == 1)
+	      {
+		if (MTYPE (m) == REAL)
+		  d_val = MAT (m, 1, 1);
+		else if (MTYPE (m) == COMPLEX)
+		  d_val = MATr (m, 1, 1);
+		else
+		  error_1 ("string matrix not allowed as conditional", 0);
+	      }
+	      else if (MNR (m) == 0 && MNC (m) == 0)
+	      {
+		/* Empty matrix tests FALSE */
+		d_val = 0.0;
+	      }
+	      else
+		error_1 ("N > 1, matrix not allowed as conditional", 0);
+	      break;
+	    }
+
+	  default:
+	    error_1 ("Illegal object for conditional", 0);
+	  }
+	  remove_tmp_destroy (d.u.ent);		/* Clean up garbage list */
+	  break;
+	}
+
+	/*
+	 * Finally, do the jump if called for.
+	 */
+
+	if (d_val)
+	  pc++;
+	else
+	  pc = pc + (ijmp);
+      }
       break;
 
-    case OP_WHILE:
-      whilecode ();
+    case OP_FOR_LOOP:
+      {
+	/* Get the current for-loop */
+	ForLoop *fl;
+	ListNode *var;
+	ListNode *flnode = list_GetFirstNode (flstack);
+
+	fl = (ForLoop *) e_data (flnode);
+	var = (ListNode *) fl->var;
+
+	/* Check for end-of-loop. */
+	if (fl->cnt < fl->n)
+	{
+	  /* Assign the proper value to the loop-variable. */
+	  switch (fl->type)
+	  {
+	  case NUM_REAL:
+	    SVALr (e_data (var)) = MATrv (fl->v, fl->cnt++);
+	    break;
+
+	  case NUM_COMPLEX:
+	    SVALr (e_data (var)) = MATcvr (fl->v, fl->cnt);
+	    SVALi (e_data (var)) = MATcvi (fl->v, fl->cnt++);
+	    break;
+
+	  case STRING:
+	    string_Set (e_data (var), cpstr (MATsv (fl->v, fl->cnt++)));
+	    break;
+	  }
+
+	  /* Jump back to top of loop. */
+	  pc = pc - fl->jmp;
+	}
+
+	/* else continue on to OP_FOR_LOOP_DONE */
+      }
       break;
 
     case OP_SWAP:
@@ -1372,16 +1686,10 @@ execute (p)
       break;
 
     case OP_POP:
-      if (stackp <= stack)
-      {
-	error_1 ("pop()", "stack underflow");
-      }
       --stackp;
       break;
 
     case OP_POP_CLEAN:
-      if (stackp <= stack)
-	error_1 ("pop()", "stack underflow");
       --stackp;
       if ((*stackp).type == ENTITY)
       {
@@ -1544,14 +1852,6 @@ execute (p)
       push (new);
       break;
 
-    case OP_BREAK:
-      breaking = 1;
-      break;
-
-    case OP_CONTINUE:
-      continuing = 1;
-      break;
-
     case OP_QUIT:
       quit_code ();
       break;
@@ -1596,6 +1896,36 @@ execute (p)
       rfile_load (cpstr ((*pc++).ptr));
       break;
 
+    case OP_REQ_NAME:
+      require (cpstr ((*pc++).ptr));
+      break;
+
+    case OP_OLIST_ASSIGN:
+      /*
+       * Pop the RHS (the list), N, and
+       * the N LHS entities.
+       * Do the assignment.
+       */
+
+      d = *--stackp;
+      d1 = *--stackp;
+      /* Let olist_assign() do the rest of the popping. */
+      new = olist_assign (d, d1);
+      *stackp++ = new;
+      break;
+      
+    case OP_OLIST:
+      /*
+       * All the elements are already on the stack, 
+       * simply push N.
+       */
+
+      n = (*pc++).op_code;
+      new.type = CONSTANT;
+      new.u.val = (double) n;
+      *stackp++ = new;
+      break;
+
     case OP_HELP:
       help ();
       break;
@@ -1724,21 +2054,12 @@ assign (d1, d2)
    * Check for bad LHS values.
    */
 
-  if (d1.type == ENTITY)
+  if (e_type (d1.u.ent) == BLTIN)
+    error_1 ("cannot assign to a built-in function", 0);
+  else if (e_type (d1.u.ent) == BTREE)
   {
-    if (e_type (d1.u.ent) == SCALAR || e_type (d1.u.ent) == MATRIX)
-      ;
-    else if (e_type (d1.u.ent) == BLTIN)
-      error_1 ("cannot assign to a built-in function", 0);
-    else if (e_type (d1.u.ent) == BTREE)
-    {
-      if (!strncmp (e_name (d1.u.ent), "$$", 2))
-	error_1 ("cannot destroy global symbol table", 0);
-    }
-  }
-  else if ((d1.type == CONSTANT) || (d1.type == iCONSTANT))
-  {
-    error_1 ("cannot assign a new value to a constant", 0);
+    if (!strncmp (e_name (d1.u.ent), "$$", 2))
+      error_1 ("cannot destroy global symbol table", 0);
   }
 
   /*
@@ -2014,7 +2335,7 @@ bltin (sp, n_arg, popf)
     d_args[i] = pop ();
 
   if (popf)
-    extern_pop();
+    extern_pop ();
 
   /* call built-in function */
   (*built_in_func) (&return_ptr, n_arg, d_args);
@@ -2048,458 +2369,13 @@ bltin (sp, n_arg, popf)
   }
 }
 
-/* **************************************************************
- * Evaluate while construct
- * ************************************************************** */
-void
-whilecode ()
-{
-  double d_val = 0.0;
-  Datum d;
-  Inst *bodyptr, *endptr, *condptr;
-  Matrix *m;
-
-  /* Calc ptrs to various parts of stmt using offsets */
-  bodyptr = (pc - 1) + (*pc).op_code;
-  endptr = (pc - 1) + (*(pc + 1)).op_code;
-  condptr = pc + 2;
-
-  execute (condptr);		/* condition */
-  d = pop ();
-
-  /* Eval while argument */
-  switch (d.type)
-  {
-  case CONSTANT:
-  case iCONSTANT:
-    d_val = d.u.val;
-    break;
-  case ENTITY:
-    switch (e_type (d.u.ent))
-    {
-    case SCALAR:
-      d_val = SVALr (e_data (d.u.ent));
-      break;
-    case MATRIX:
-      m = (Matrix *) e_data (d.u.ent);
-      if (MNR (m) == 1 && MNC (m) == 1)
-      {
-	if (MTYPE (m) == REAL)
-	  d_val = MAT (m, 1, 1);
-	else if (MTYPE (m) == COMPLEX)
-	  d_val = MATr (m, 1, 1);
-	else
-	  error_1 ("string matrix not allowed as while() argument", 0);
-      }
-      else if (MNR (m) == 0 && MNC (m) == 0)
-      {
-	d_val = 0.0;
-      }
-      else
-	error_1 ("N > 1, matrix not allowed as while() argument", 0);
-      break;
-    default:
-      error_1 ("Illegal object for while() argument", 0);
-    }
-    remove_tmp_destroy (d.u.ent);	/* Clean up garbage list */
-    break;
-  }
-
-  while (d_val)
-  {
-    execute (bodyptr);		/* loop body */
-    if (returning)
-      break;
-    if (breaking)
-    {
-      breaking = 0;
-      break;
-    }
-    if (continuing)
-      continuing = 0;
-
-    execute (condptr);		/* condition */
-    d = pop ();
-
-
-    /* Eval while argument */
-    switch (d.type)
-    {
-    case CONSTANT:
-    case iCONSTANT:
-      d_val = d.u.val;
-      break;
-    case ENTITY:
-      switch (e_type (d.u.ent))
-      {
-      case SCALAR:
-	d_val = SVALr (e_data (d.u.ent));
-	break;
-      case MATRIX:
-	m = (Matrix *) e_data (d.u.ent);
-	if (MNR (m) == 1 && MNC (m) == 1)
-	{
-	  if (MTYPE (m) == REAL)
-	    d_val = MAT (m, 1, 1);
-	  else if (MTYPE (m) == COMPLEX)
-	    d_val = MATr (m, 1, 1);
-	  else
-	    error_1 ("string matrix not allowed as while() argument", 0);
-	}
-	else if (MNR (m) == 0 && MNC (m) == 0)
-	{
-	  d_val = 0.0;
-	}
-	else
-	  error_1 ("N > 1, matrix not allowed as while() argument", 0);
-	break;
-      default:
-	error_1 ("Illegal object for while() argument", 0);
-      }
-      remove_tmp_destroy (d.u.ent);	/* Clean up garbage list */
-      break;
-    }
-  }
-
-  if (!returning)
-    pc = endptr;		/* next statement */
-}
-
-/* **************************************************************
- * Evaluate for construct.
- * *(savepc+1) == offset to body of loop
- * *(savepc+2) == offset end of loop
- * Form for loop, loop over for_loop_body length(vec_expr)
- * times. Set the value of var to the value in the ith
- * element of vec_expr.
- *
- * for ( var in vec_expr ) { body... }
- *
- * ************************************************************** */
-void
-forcode ()
-{
-  int i, limit;
-  Datum d_var, d_vec_expr;
-  ListNode *var, *vec_expr;
-  Inst *bodyptr, *endptr, *condptr;
-
-  limit = 0;
-  vec_expr = 0;
-  bodyptr = (pc - 1) + (*pc).op_code;
-  endptr = (pc - 1) + (*(pc + 1)).op_code;
-  condptr = pc + 2;
-
-  /* execute code untill loop body */
-  execute (condptr);
-
-  /*
-   * Then pop top two elements off of the stack.
-   * These are the var, and the vec_expr
-   * from:  for( var in vec_expr )
-   */
-
-  d_vec_expr = pop ();
-  d_var = pop ();
-
-  /* get var name, address */
-  var = d_var.u.ent;
-
-  /*
-   * Get vector_expression entity.
-   * Make sure it is an ENTITY.
-   */
-
-  if (d_vec_expr.type == ENTITY)
-    vec_expr = d_vec_expr.u.ent;
-  else if (d_vec_expr.type == CONSTANT)
-  {
-    vec_expr = install_tmp (SCALAR,
-			    scalar_Create (d_vec_expr.u.val),
-			    scalar_Destroy);
-  }
-  else if (d_vec_expr.type == iCONSTANT)
-  {
-    vec_expr = install_tmp (SCALAR,
-			    scalar_CreateC (0.0, d_vec_expr.u.val),
-			    scalar_Destroy);
-  }
-
-  /*
-   * Now check the ENTITY for proper markings.
-   */
-
-  if (listNode_GetKey (vec_expr) != 0)
-  {
-    /*
-     * Copy the vector. We have to do this in case the program
-     * changes the vector in mid-loop.
-     */
-    ListNode *tmp = d_vec_expr.u.ent;
-    if (e_type (tmp) == MATRIX)
-    {
-      vec_expr = install_tmp (MATRIX,
-			      matrix_Copy (e_data (tmp)),
-			      matrix_Destroy);
-    }
-    else if (e_type (tmp) == SCALAR)
-    {
-      vec_expr = install_tmp (SCALAR,
-			      scalar_CreateC (SVALr (e_data (tmp)),
-					      SVALi (e_data (tmp))),
-			      scalar_Destroy);
-    }
-    else if (e_type (tmp) == STRING)
-    {
-      /* Make this into a 1-by-1 string matrix */
-      Matrix *mtmp;
-      mtmp = matrix_CreateS (1, 1);
-      MATs (mtmp, 1, 1) = cpstr (string_GetString (e_data (tmp)));
-      vec_expr = install_tmp (MATRIX, mtmp, matrix_Destroy);
-    }
-    else
-      error_1 (e_name (tmp), "invalid type for-loop vector");
-  }
-
-  /*
-   * Set the loop vector's name so it doesn't get cleaned.
-   * Start the name with something users can't use.
-   */
-
-  listNode_SetKey (vec_expr, cpstr ("-LOOP-VECTOR"));
-
-  /*
-   * vec_expr can be a scalar or a vector.
-   * Get the loop limit.
-   */
-
-  if (e_type (vec_expr) == MATRIX)
-    limit = (int) (MNR (e_data (vec_expr)) * MNC (e_data (vec_expr)));
-  else if (e_type (vec_expr) == SCALAR)
-    limit = 1;
-  else if (e_type (vec_expr) == STRING)
-    limit = 1;
-  else
-    error_1 ("invalid vector expression used in for statement", 0);
-
-  /* Check limit for no-execution condition */
-  if (limit == 0)
-  {
-    pc = endptr;
-    return;
-  }
-
-  /*
-   * If the vector_expr evaluates to a SCALAR, then
-   * execute the loop once with the var set to the
-   * scalar value.
-   */
-
-  if (e_type (vec_expr) == SCALAR)
-  {
-    /* force var to be a scalar */
-    if (e_type (var) != SCALAR)
-    {
-      listNode_DestroyDataOnly (var);
-      listNode_AttachData (var, SCALAR, scalar_Create (0.0), scalar_Destroy);
-      scalar_SetName (e_data (var), cpstr (e_name (var)));
-    }
-
-    for (i = limit; i <= limit; i++)
-    {
-      scalar_SetValC (e_data (var), SVALr (e_data (vec_expr)),
-		      SVALi (e_data (vec_expr)));
-      execute (bodyptr);	/* loop body */
-      if (returning)
-	break;
-      if (breaking)
-      {
-	breaking = 0;
-	break;
-      }
-      if (continuing)
-	continuing = 0;
-    }
-  }
-  else
-  {
-    /* Normal execution of for loop */
-
-    if (MTYPE (e_data (vec_expr)) == REAL)
-    {
-      /* force var to be a scalar */
-      if (e_type (var) != SCALAR)
-      {
-	listNode_DestroyDataOnly (var);
-	listNode_AttachData (var, SCALAR, scalar_Create (0.0), scalar_Destroy);
-	scalar_SetName (e_data (var), cpstr (e_name (var)));
-      }
-      for (i = 1; i <= limit; i++)
-      {
-	SVALr (e_data (var)) = MATrv (e_data (vec_expr), i - 1);
-
-	execute (bodyptr);	/* loop body */
-	if (returning)
-	  break;
-	if (breaking)
-	{
-	  breaking = 0;
-	  break;
-	}
-	if (continuing)
-	  continuing = 0;
-      }
-    }
-    else if (MTYPE (e_data (vec_expr)) == COMPLEX)
-    {
-      /* force var to be a scalar */
-      if (e_type (var) != SCALAR)
-      {
-	listNode_DestroyDataOnly (var);
-	listNode_AttachData (var, SCALAR, scalar_Create (0.0), scalar_Destroy);
-	scalar_SetName (e_data (var), cpstr (e_name (var)));
-      }
-      for (i = 1; i <= limit; i++)
-      {
-	SVALr (e_data (var)) = MATcvr (e_data (vec_expr), i - 1);
-	SVALi (e_data (var)) = MATcvi (e_data (vec_expr), i - 1);
-
-	execute (bodyptr);	/* loop body */
-	if (returning)
-	  break;
-	if (breaking)
-	{
-	  breaking = 0;
-	  break;
-	}
-	if (continuing)
-	  continuing = 0;
-      }
-    }
-    else if (MTYPE (e_data (vec_expr)) == STRING)
-    {
-      /* force var to be a string */
-      if (e_type (var) != STRING)
-      {
-	listNode_DestroyDataOnly (var);
-	listNode_AttachData (var, STRING, string_Create (0), string_Destroy);
-	string_SetName (e_data (var), cpstr (e_name (var)));
-      }
-      for (i = 1; i <= limit; i++)
-      {
-	string_Set (e_data (var),
-		    cpstr (MATsv (e_data (vec_expr), i - 1)));
-	execute (bodyptr);	/* loop body */
-	if (returning)
-	  break;
-	if (breaking)
-	{
-	  breaking = 0;
-	  break;
-	}
-	if (continuing)
-	  continuing = 0;
-      }
-    }
-  }
-
-  /* Now destroy the loop-vector */
-  listNode_SetKey (vec_expr, 0);
-  remove_tmp_destroy (vec_expr);
-
-  if (!returning)
-    pc = endptr;
-}
-
-/* **************************************************************
- * Evaluate if construct
- * ************************************************************** */
-void
-ifcode ()
-{
-  double d_val = 0.0;
-  Datum d;
-  Inst *bodyptr, *endptr, *condptr, *elseptr;
-  Matrix *m;
-
-  /* Set up ptrs and do a little error checking */
-  if ((*pc).op_code == 0)
-    error_1 ("Invalid ifcode offset", 0);
-  bodyptr = (pc - 1) + (*pc).op_code;
-
-  if ((*(pc + 1)).op_code)
-    elseptr = (pc - 1) + (*(pc + 1)).op_code;
-  else
-    elseptr = (Inst *) 0;
-
-  if ((*(pc + 2)).op_code == 0)
-    error_1 ("Invalid ifcode offset", 0);
-  endptr = (pc - 1) + (*(pc + 2)).op_code;
-  condptr = pc + 3;
-
-  /*Inst *savepc = pc; */
-
-  execute (condptr);		/* condition */
-  d = pop ();
-
-  /*
-   * We can only eval scalars and constants in a conditional stmt.
-   */
-  switch (d.type)
-  {
-  case CONSTANT:
-  case iCONSTANT:
-    d_val = d.u.val;
-    break;
-  case ENTITY:
-    switch (e_type (d.u.ent))
-    {
-    case SCALAR:
-      d_val = SVALr (e_data (d.u.ent));
-      break;
-    case MATRIX:
-      m = (Matrix *) e_data (d.u.ent);
-      if (MNR (m) == 1 && MNC (m) == 1)
-      {
-	if (MTYPE (m) == REAL)
-	  d_val = MAT (m, 1, 1);
-	else if (MTYPE (m) == COMPLEX)
-	  d_val = MATr (m, 1, 1);
-	else
-	  error_1 ("string matrix not allowed as if() argument", 0);
-      }
-      else if (MNR (m) == 0 && MNC (m) == 0)
-      {
-	d_val = 0.0;
-      }
-      else
-	error_1 ("N > 1, matrix not allowed as if() argument", 0);
-      break;
-    default:
-      error_1 ("Illegal object for if() argument", 0);
-    }
-    remove_tmp_destroy (d.u.ent);	/* Clean up garbage list */
-    break;
-  }
-
-  if (d_val)
-    execute (bodyptr);
-  else if (elseptr)		/* else part */
-    execute (elseptr);
-
-  /* If we've seen any of the following leave the code pointer
-     pointing to a STOP */
-  if (!returning && !breaking && !continuing)
-    pc = endptr;		/* next stmt */
-}
-
 /* **************************************************************
  * Create an list. The next instruction contains the number of
  * items on the stack that we need to install on the new list.
  * Either we are moving ptrs, as in: NAME '=' vec_expr, or we
  * are copying the data as in: vec_expr. There are three possibilities
  * for listNode keys...
- * "-"      Result of NAME '=' vec_expr: Use NAME as index
+ * "-L-"    Result of NAME '=' vec_expr: Use NAME as index
  * NULL ptr Result of vec_expr: copy and assign numerical index
  * NAME     Result of vec_expr: copy and assign numerical index
  * ************************************************************** */
@@ -2541,7 +2417,7 @@ list_create ()
     switch (e_type (d.u.ent))
     {
     case SCALAR:
-      if (e_name (d.u.ent) == 0 || strcmp ("-", e_name (d.u.ent)))
+      if (e_name (d.u.ent) == 0 || strcmp ("-L-", e_name (d.u.ent)))
       {
 	sprintf (jstr, "%d", j--);
 	lnode = install (e_data (new_list.u.ent),
@@ -2560,7 +2436,7 @@ list_create ()
       }
       break;
     case MATRIX:
-      if (e_name (d.u.ent) == 0 || strcmp ("-", e_name (d.u.ent)))
+      if (e_name (d.u.ent) == 0 || strcmp ("-L-", e_name (d.u.ent)))
       {
 	sprintf (jstr, "%d", j--);
 	lnode = install (e_data (new_list.u.ent),
@@ -2579,7 +2455,7 @@ list_create ()
       }
       break;
     case STRING:
-      if (e_name (d.u.ent) == 0 || strcmp ("-", e_name (d.u.ent)))
+      if (e_name (d.u.ent) == 0 || strcmp ("-L-", e_name (d.u.ent)))
       {
 	sprintf (jstr, "%d", j--);
 	lnode = install (e_data (new_list.u.ent),
@@ -2598,7 +2474,7 @@ list_create ()
       }
       break;
     case BTREE:
-      if (e_name (d.u.ent) == 0 || strcmp ("-", e_name (d.u.ent)))
+      if (e_name (d.u.ent) == 0 || strcmp ("-L-", e_name (d.u.ent)))
       {
 	sprintf (jstr, "%d", j--);
 	lnode = install (e_data (new_list.u.ent),
@@ -2617,7 +2493,7 @@ list_create ()
       }
       break;
     case U_FUNCTION:
-      if (e_name (d.u.ent) == 0 || strcmp ("-", e_name (d.u.ent)))
+      if (e_name (d.u.ent) == 0 || strcmp ("-L-", e_name (d.u.ent)))
       {
 	sprintf (jstr, "%d", j--);
 	lnode = install (e_data (new_list.u.ent),
@@ -2929,7 +2805,7 @@ list_member ()
 	if (e_name (vid.u.ent) == 0)
 	{
 	  ListNode *ltmp = listNode_Create ();
-	  ltmp = install_tmp (e_type (new.u.ent), e_data (new.u.ent), 
+	  ltmp = install_tmp (e_type (new.u.ent), e_data (new.u.ent),
 			      e_dtor (new.u.ent));
 	  e_data (new.u.ent) = 0;
 	  e_dtor (new.u.ent) = 0;
@@ -3103,7 +2979,6 @@ list_assign ()
   push (vid_d);
 }
 
-
 /* **************************************************************
  * Given: NAME '=' vec_expr
  * Copy the vec_expr into a ListNode with name NAME, and push
@@ -3178,7 +3053,7 @@ list_el_create ()
    * Give the new element a UNIQUE name, so we can identify
    * it in list_create().
    */
-  listNode_SetKey (lnode, cpstr ("-"));
+  listNode_SetKey (lnode, cpstr ("-L-"));
 
   /* Push the new object on the stack */
   rhs.u.ent = lnode;
@@ -3186,6 +3061,103 @@ list_el_create ()
   push (rhs);
 }
 
+/*
+ * < s1; s2; s3; ... > = LIST;
+ *
+ * Return the s1 entity to pop back on the stack ?
+ */
+
+Datum
+olist_assign (drhs, dN)
+     Datum drhs, dN;
+{
+  char **rhs_names;
+  int N, i, nrhs;
+  Btree *rhs;
+  Datum d1, d2, new;
+  ListNode *ltmp;
+
+  N = 0; rhs = 0;   /* Initialize */
+
+  /* Check N. */
+  if (dN.type == CONSTANT)
+  {
+    N = (int) dN.u.val;
+  }
+  else
+  {
+    error_1 ("olist_assign: terrible error", 0);
+  }
+
+  /*
+   * Now loop N times, popping the stack, and 
+   * doing the assignment. If there are more RHS
+   * elements than LHS, that is OK. If there are
+   * more LHS, than RHS, error.
+   */
+
+  /* Check rhs type. */
+  if (drhs.type == ENTITY)
+  {
+    if (e_type (drhs.u.ent) == BTREE)
+    {
+      rhs = (Btree *) e_data (drhs.u.ent);
+    }
+    else
+      error_1 (e_name (drhs.u.ent), "invalid RHS for open-list assign");
+  }
+  else
+    error_1 ("invalid RHS for open-list assign", 0);
+
+  /* Check N-rhs and N-lhs. */
+  if ((nrhs = btree_GetNumNodes (rhs)) >= N)
+  {
+    /* OK */ ;
+  }
+  else
+    error_1 (e_name (drhs.u.ent), "RHS length must be >= LHS length");
+
+  /* 
+   * Get the names in the RHS tree, so that we can extract the
+   * elements alphabetically.
+   */
+
+  rhs_names = (char **) MALLOC (nrhs * sizeof (char *));
+  get_btree_node_names (rhs->root_node, rhs_names);
+
+  /*
+   * Now loop over the list(s) and do the assigns.
+   */
+
+  for (i = N-1; i >= 0; i--)
+  {
+    d1 = pop ();
+    ltmp = btree_FindNode (rhs, rhs_names[i]);
+
+    d2.u.ent = ltmp;
+    d2.type = ENTITY;
+
+    /*
+     * We must use assign(), and do a copy, cause
+     * we don't know if drhs is a function return or
+     * a regular list.
+     */
+
+    new = assign (d1, d2);
+  }
+
+  /* Clean Up. */
+
+  for (i = 0; i < nrhs; i++)
+    FREE (rhs_names[i]);
+  FREE (rhs_names);
+
+  if (e_name (drhs.u.ent) == 0)
+    remove_tmp_destroy (drhs.u.ent);
+
+  return (new);
+}
+
 /* **************************************************************
  * Find the line number ascociated with the current error. Back
  * up from the current program instruction untill we find a
