@@ -1837,8 +1837,8 @@ matrix_Eig_GNEP (MA, MB, val, vecr)
 }
 
 void
-matrix_Eig_GNEPa (MA, MB, valpha, vbeta, vecl, vecr)
-     Matrix *MA, *MB, **valpha, **vbeta, **vecl, **vecr;
+matrix_Eig_GNEPa (MA, MB, valpha, vbeta, vecr, vecl)
+     Matrix *MA, *MB, **valpha, **vbeta, **vecr, **vecl;
 {
   ASSERT (MA);
   ASSERT (MB);
@@ -1966,8 +1966,8 @@ matrix_Eig_GNEPa (MA, MB, valpha, vbeta, vecl, vecr)
       
       for (k = 0; k < n; k++)
       {
-	MATr0 (eval, 1, k) = MAT0 (alphar, 1, k);
-	MATi0 (eval, 1, k) = MAT0 (alphai, 1, k);
+	MATr0 (eval, 0, k) = MAT0 (alphar, 0, k);
+	MATi0 (eval, 0, k) = MAT0 (alphai, 0, k);
 
 	if (MATrv (alphai, k) == 0.0)
 	{
