@@ -350,6 +350,7 @@ list_EnqueueNode (list, nodeToInsert)
  *          the first ListNode in the List.
  *
  * ************************************************************************ */
+
 ListNode *
 list_DequeueNode (list)
      List *list;
@@ -362,6 +363,7 @@ list_DequeueNode (list)
  * Push a ListNode onto the front of a List. Do Not use list_InsertNode,
  * we want this to be faster, so we are more specific.
  * ************************************************************************ */
+
 ListNode *
 list_PushNode (list, pnode)
      List *list;
@@ -389,14 +391,16 @@ list_PushNode (list, pnode)
 
 /* ************************************************************************
  * PURPOSE: Pop a ListNode from a List.  Popping a ListNode, means
- *          detaching the last ListNode in the List.
+ *          detaching the FIRST ListNode in the List. Push node puts
+ *          on the front of the list, and pop removes from the front.
  * ************************************************************************ */
+
 ListNode *
 list_PopNode (list)
      List *list;
 {
   ASSERT (list);
-  return (list_DetachNodeByPos (list, list_GetNumNodes (list)));
+  return (list_DetachNodeByPos (list, 1));
 }
 
 /* ************************************************************************
