@@ -61,7 +61,7 @@ static void think_c_setup _PROTO ((void));
 
 #endif /* THINK_C */
 
-#ifdef HAVE_DIRENT
+#ifdef HAVE_DIRENT_H
 #include <dirent.h>
 #else
 #include <sys/dir.h>
@@ -89,6 +89,7 @@ int rfile_dir _PROTO ((char *dirname));
 /* code.c */
 extern void init_frame _PROTO ((void));
 extern void init_stack _PROTO ((void));
+extern void init_flstack _PROTO ((void));
 
 void init_misc _PROTO ((void));
 void init_var _PROTO ((void));
@@ -266,6 +267,9 @@ main (argc, argv)
 /* **************************************************************
  * Misc initialization.
  * ************************************************************** */
+
+#include "fpe.h"
+
 void
 init_misc ()
 {
@@ -275,10 +279,10 @@ init_misc ()
   /* Initialize the interpreter stacks */
   init_frame ();
   init_stack ();
+  init_flstack ();
 
-  /* Probabaly a vain attempt to deal with fpe exceptions */
-  FP_SET_MASK;
-  /* fpsetmask (FP_X_INV | FP_X_DZ | FP_X_OFL); */
+  /* Try and setup for fpe exceptions. */
+  setup_fpe_handling ();
 }
 
 /* **************************************************************
@@ -400,6 +404,7 @@ extern Inst *get_program_counter _PROTO ((void));
 extern void set_program_counter _PROTO ((Inst * prgm));
 extern int reset_frame_ptr _PROTO ((void));
 extern int reset_stack_ptr _PROTO ((void));
+extern int clean_up_bc_lists _PROTO ((void));
 extern Datum *get_stackp _PROTO ((void));
 extern void set_stackp _PROTO ((Datum * new_stackp));
 
@@ -456,6 +461,7 @@ run_program (input)
       /* An error (longjmp) has occurred */
       reset_frame_ptr ();
       reset_stack_ptr ();
+      clean_up_bc_lists ();
       retval = -1;
       if (input == 0)
 	continue;
@@ -673,7 +679,7 @@ rfile_dir (dirname)
   /* FIX */
   char tmp[100];
   DIR *dirp;
-#ifdef HAVE_DIRENT
+#ifdef HAVE_DIRENT_H
   struct dirent *direntp;
 #else
   struct direct *direntp;
@@ -712,7 +718,7 @@ print_greeting ()
 {
   printf ("Welcome to RLaB. New users type `help INTRO'\n");
   printf ("RLaB version %s ", version_string);
-  printf ("Copyright (C) 1992, 93, 94 Ian Searle\n");
+  printf ("Copyright (C) 1992-95 Ian Searle\n");
   printf ("RLaB comes with ABSOLUTELY NO WARRANTY; for details type");
   printf (" `help WARRANTY'\n");
   printf ("This is free software, and you are welcome to redistribute it");
