@@ -266,7 +266,15 @@ Norm (return_ptr, n_args, d_arg)
     else
     {
       /* Compute a P-Norm */
-      p = (double) MAT (e_data (M2), 1, 1);      
+      p = (double) MAT (e_data (M2), 1, 1);
+      if (detect_inf_r (MDPTRr (e_data (M2)), 1))
+      {
+	d = matrix_Norm (e_data (M1), "i");
+	remove_tmp_destroy (M1);
+	remove_tmp_destroy (M2);
+	*return_ptr = (VPTR) scalar_Create (d);
+	return;
+      }
       m = (Matrix *) e_data (M1);
       if (MNR (m) != 1 && MNC (m) != 1)
       {
@@ -416,7 +424,7 @@ Balance (return_ptr, n_args, d_arg)
   btree = btree_Create ();
   install (btree, cpstr ("ab"), MATRIX, Ab);
   install (btree, cpstr ("t"), MATRIX, t);
-  matrix_SetName (Ab, cpstr ("Ab"));
+  matrix_SetName (Ab, cpstr ("ab"));
   matrix_SetName (t, cpstr ("t"));
   *return_ptr = (VPTR) btree;
 
