@@ -411,6 +411,27 @@ Sum (return_ptr, n_args, d_arg)
   return;
 }
 
+void
+CumSum (return_ptr, n_args, d_arg)
+     VPTR *return_ptr;
+     int n_args;
+     Datum *d_arg;
+{
+  ListNode *M;
+
+  /* Check n_args */
+  if (n_args != 1)
+  {
+    clean_bltin_args (d_arg, n_args);
+    error_1 ("cumsum: 1 argument allowed", 0);
+  }
+
+  M = bltin_get_numeric_matrix ("cumsum", d_arg, 1);
+  *return_ptr = (VPTR) matrix_CumSum (e_data (M));
+  remove_tmp_destroy (M);
+  return;
+}
+
 void
 Prod (return_ptr, n_args, d_arg)
      VPTR *return_ptr;
@@ -429,6 +450,24 @@ Prod (return_ptr, n_args, d_arg)
   return;
 }
 
+void
+CumProd (return_ptr, n_args, d_arg)
+     VPTR *return_ptr;
+     int n_args;
+     Datum *d_arg;
+{
+  ListNode *M;
+
+  /* Check n_args */
+  if (n_args != 1)
+    error_1 ("cumprod: 1 argument allowed", 0);
+
+  M = bltin_get_numeric_matrix ("cumprod", d_arg, 1);
+  *return_ptr = (VPTR) matrix_CumProd (e_data (M));
+  remove_tmp_destroy (M);
+  return;
+}
+
 /* **************************************************************
  * Factor, and Backsub functions.
  * ************************************************************** */
