@@ -29,6 +29,7 @@
 #include "mathl.h"
 #include "util.h"
 #include "mem.h"
+#include "symbol.h"
 
 #include "bl.h"
 #include "fi.h"
@@ -1256,6 +1257,80 @@ matrix_Log10 (m)
   }
 }
 
+#ifdef HAVE_LOGB
+
+Matrix *
+matrix_Logb (m)
+     Matrix *m;
+{
+  ASSERT (m);
+  {
+    Matrix *new = 0;
+    int i;
+
+    if (MTYPE (m) == REAL)
+    {
+      /* Only allowable solution, for now. */
+      
+      new = matrix_Create (MNR (m), MNC (m));
+      for (i = 0; i < MNR (m) * MNC (m); i++)
+      {
+	MATrv (new, i) = errcheck (logb (MATrv (m, i)), "logb");
+      }
+    }
+    else
+    {
+      matrix_screen_string (m);
+      error_1 ("invalid matrix type for logb", 0);
+    }
+    return (new);
+  }
+}
+
+#endif  /* HAVE_LOGB */
+
+#ifdef HAVE_FREXP
+
+void
+matrix_Frexp (m, F, E)
+     Matrix *m, **F, **E;
+{
+  ASSERT (m);
+  {
+    ListNode *t1, *t2;
+    Matrix *e, *f;
+    int i, ie;
+
+    if (MTYPE (m) == REAL)
+    {
+      /* Only allowable solution. */
+      
+      t1 = install_tmp (MATRIX, f = matrix_Create (MNR (m), MNC (m)),
+			matrix_Destroy);
+      t2 = install_tmp (MATRIX, e = matrix_Create (MNR (m), MNC (m)),
+			matrix_Destroy);
+
+      for (i = 0; i < MNR (m) * MNC (m); i++)
+      {
+	MATrv (f, i) = errcheck (frexp (MATrv (m, i), &ie), "frexp");
+	MATrv (e, i) = (double) ie;
+      }
+      *E = e;
+      *F = f;
+      remove_tmp (t1);
+      remove_tmp (t2);
+    }
+    else
+    {
+      matrix_screen_string (m);
+      error_1 ("invalid matrix type for frexp", 0);
+    }
+    return;
+  }
+}
+
+#endif  /* HAVE_FREXP */
+
 Matrix *
 matrix_Exp (m)
      Matrix *m;
@@ -2024,6 +2099,21 @@ matrix_eq (m1, m2)
       }
       return (new);
     }
+    else if (MNR (m1) == 0 || MNR (m2) == 0)
+    {
+      if (MNR (m1) == 0 && MNR (m2) == 0)
+      {
+	new = matrix_Create (1, 1);
+	MATrv (new, 0) = 1.0;
+	return (new);
+      }
+      else
+      {
+	new = matrix_Create (1, 1);
+	MATrv (new, 0) = 0.0;
+	return (new);
+      }	
+    }
 
     /*
      * Back to regular programming.
@@ -2144,6 +2234,21 @@ matrix_ne (m1, m2)
       }
       return (new);
     }
+    else if (MNR (m1) == 0 || MNR (m2) == 0)
+    {
+      if (MNR (m1) == 0 && MNR (m2) == 0)
+      {
+	new = matrix_Create (1, 1);
+	MATrv (new, 0) = 0.0;
+	return (new);
+      }
+      else
+      {
+	new = matrix_Create (1, 1);
+	MATrv (new, 0) = 1.0;
+	return (new);
+      }	
+    }
 
     /*
      * Back to regular programming.
@@ -2876,6 +2981,15 @@ matrix_scalar_eq (m, s)
   {
     register int i;
     Matrix *new;
+
+    if (MNR (m) == 0 && MNC (m) == 0)
+    {
+      /* Special case, no scalar is equal to the empty matrix. */
+      new = matrix_Create (1, 1);
+      MATrv (new, 0) = 0.0;
+      return (new);
+    }
+
     if ((MTYPE (m) == REAL) && (SVALi (s) == 0.0))
     {
       /* Most common operation */
@@ -2914,6 +3028,15 @@ matrix_scalar_ne (m, s)
   {
     register int i;
     Matrix *new;
+
+    if (MNR (m) == 0 && MNC (m) == 0)
+    {
+      /* Special case, no scalar is equal to the empty matrix. */
+      new = matrix_Create (1, 1);
+      MATrv (new, 0) = 1.0;
+      return (new);
+    }
+
     if ((MTYPE (m) == REAL) && (SVALi (s) == 0.0))
     {
       /* Most common operation */
