@@ -1585,6 +1585,87 @@ matrix_Sum (m)
   }
 }
 
+Matrix *
+matrix_CumSum (m)
+     Matrix *m;
+{
+  ASSERT (m);
+  {
+    int i, j;
+    Matrix *new = 0;
+
+    if (MTYPE (m) == REAL)
+    {
+      if (MNR (m) == 1 || MNC (m) == 1)
+      {
+	int N = MNR (m) * MNC (m);
+	new = matrix_Create (MNR (m), MNC (m));
+	MATrv (new, 0) = MATrv (m, 0);
+	for (i = 1; i < N; i++)
+	{
+	  MATrv (new, i) = MATrv (new, i-1) + MATrv (m, i);
+	}
+      }
+      else
+      {
+	new = matrix_Create (MNR (m), MNC (m));
+	/* Initialize first row. */
+	for (i = 0; i < MNC (m); i++)
+	  MAT0 (new, 0, i) = MAT0 (m, 0, i);
+
+	/* Now compute running sum. */
+	for (i = 0; i < MNC (m); i++)
+	{
+	  for (j = 1; j < MNR (m); j++)
+	  {
+	    MAT0 (new, j, i) = MAT0 (new, j-1, i) + MAT0 (m, j, i);
+	  }
+	}
+      }
+    }
+    else if (MTYPE (m) == COMPLEX)
+    {
+      if (MNR (m) == 1 || MNC (m) == 1)
+      {
+	int N = MNR (m) * MNC (m);
+	new = matrix_CreateC (MNR (m), MNC (m));
+	MATcvr (new, 0) = MATcvr (m, 0);
+	MATcvi (new, 0) = MATcvi (m, 0);
+	
+	for (i = 1; i < N; i++)
+	{
+	  MATcvr (new, i) = MATcvr (new, i-1) + MATcvr (m, i);
+	  MATcvi (new, i) = MATcvi (new, i-1) + MATcvi (m, i);
+	}
+      }
+      else
+      {
+	new = matrix_CreateC (MNR (m), MNC (m));
+	/* Initialize first row. */
+	for (i = 0; i < MNC (m); i++)
+	{
+	  MATr0 (new, 0, i) = MATr0 (m, 0, i);
+	  MATi0 (new, 0, i) = MATi0 (m, 0, i);
+	}
+
+	/* Now compute running sum. */
+	for (i = 0; i < MNC (m); i++)
+	{
+	  for (j = 1; j < MNR (m); j++)
+	  {
+	    MATr0 (new, j, i) = MATr0 (new, j-1, i) + MATr0 (m, j, i);
+	    MATi0 (new, j, i) = MATi0 (new, j-1, i) + MATi0 (m, j, i);
+	  }
+	}
+      }
+    }
+    else
+      error_1 (matrix_GetName (m), "invalid type matrix for cumsum");
+
+    return (new);
+  }
+}
+
 VPTR
 matrix_Prod (m)
      Matrix *m;
@@ -1650,6 +1731,87 @@ matrix_Prod (m)
   }
 }
 
+Matrix *
+matrix_CumProd (m)
+     Matrix *m;
+{
+  ASSERT (m);
+  {
+    int i, j;
+    Matrix *new = 0;
+
+    if (MTYPE (m) == REAL)
+    {
+      if (MNR (m) == 1 || MNC (m) == 1)
+      {
+	int N = MNR (m) * MNC (m);
+	new = matrix_Create (MNR (m), MNC (m));
+	MATrv (new, 0) = MATrv (m, 0);
+	for (i = 1; i < N; i++)
+	{
+	  MATrv (new, i) = MATrv (new, i-1) * MATrv (m, i);
+	}
+      }
+      else
+      {
+	new = matrix_Create (MNR (m), MNC (m));
+	/* Initialize first row. */
+	for (i = 0; i < MNC (m); i++)
+	  MAT0 (new, 0, i) = MAT0 (m, 0, i);
+
+	/* Now compute running sum. */
+	for (i = 0; i < MNC (m); i++)
+	{
+	  for (j = 1; j < MNR (m); j++)
+	  {
+	    MAT0 (new, j, i) = MAT0 (new, j-1, i) * MAT0 (m, j, i);
+	  }
+	}
+      }
+    }
+    else if (MTYPE (m) == COMPLEX)
+    {
+      if (MNR (m) == 1 || MNC (m) == 1)
+      {
+	int N = MNR (m) * MNC (m);
+	new = matrix_CreateC (MNR (m), MNC (m));
+	MATcvr (new, 0) = MATcvr (m, 0);
+	MATcvi (new, 0) = MATcvi (m, 0);
+	
+	for (i = 1; i < N; i++)
+	{
+	  MATcv (new, i) = complex_Multiply (MATcvr (new,i-1), MATcvi (new,i-1),
+					     MATcvr (m, i), MATcvi (m, i));
+	}
+      }
+      else
+      {
+	new = matrix_CreateC (MNR (m), MNC (m));
+	/* Initialize first row. */
+	for (i = 0; i < MNC (m); i++)
+	{
+	  MATr0 (new, 0, i) = MATr0 (m, 0, i);
+	  MATi0 (new, 0, i) = MATi0 (m, 0, i);
+	}
+
+	/* Now compute running sum. */
+	for (i = 0; i < MNC (m); i++)
+	{
+	  for (j = 1; j < MNR (m); j++)
+	  {
+	    MATc0 (new,j,i) = complex_Multiply(MATr0(new,j-1,i), MATi0(new,j-1,i),
+					       MATr0(m,j,i), MATi0(m,j,i));
+	  }
+	}
+      }
+    }
+    else
+      error_1 (matrix_GetName (m), "invalid type matrix for cumprod");
+
+    return (new);
+  }
+}
+
 /*
  * matrix ^ matrix
  * Only handle 1-by-1 ^ 1-by-1 case,
