@@ -25,10 +25,28 @@
 
 # See the file ../COPYING
 
+#
+# If your system does not deal with Infs and NaNs well, then
+# uncomment the following lines.
+#
+
+# static (isinf, isnan)
+#
+# isinf = function ( A ) { return (0); };
+# isnan = function ( A ) { return (0); };
 
 static (WIN)		# The static plot window structure
 static (P)		# The active/current plot window
 
+#
+# Maintain the transformations for 3-D plots.
+#
+
+static (basex, basey, height)
+basex = 2;
+basey = 2;
+height = 4;
+
 #
 # Static (private) functions. For use from within
 # this file only.
@@ -93,7 +111,7 @@ create_plot_object = function ( N, nx, ny )
   {
     pobj.style.[i] = "line";	# The type/style of plot to draw
     pobj.pstyle[i] = 1;	        # The point-style
-    pobj.nbin[i] = inf();	# The number of bins for a histogram
+    pobj.nbin[i] = 1j;          # The number of bins for a histogram
     pobj.width[i] = 1;		# The pen width for current plot
     pobj.font[i] = 1;		# The current font
     pobj.xlabel[i] = "";
@@ -111,12 +129,12 @@ create_plot_object = function ( N, nx, ny )
     pobj.alt[i] = 60;
     pobj.az[i] = 45;
     
-    pobj.xmin[i] = inf();
-    pobj.xmax[i] = inf();
-    pobj.ymin[i] = inf();
-    pobj.ymax[i] = inf();
-    pobj.zmin[i] = inf();
-    pobj.zmax[i] = inf();
+    pobj.xmin[i] = 1j;
+    pobj.xmax[i] = 1j;
+    pobj.ymin[i] = 1j;
+    pobj.ymax[i] = 1j;
+    pobj.zmin[i] = 1j;
+    pobj.zmax[i] = 1j;
     
     pobj.page.xp = 0;
     pobj.page.yp = 0;
@@ -208,6 +226,25 @@ showpwin = function ( all )
   }
 };
 
+getplot = function ( win_no )
+{
+  local (win_no)
+
+  if (length (WIN) != 0)
+  {
+    if (!exist (win_no)) { win_no = P; }
+
+    if (exist (WIN.[win_no]))
+    {
+      return (WIN.[win_no]);
+    else
+      return 0;
+    }
+  }
+  return <<>>;
+};
+
+
 ##############################################################################
 #
 # Set/start/select the plot device
@@ -265,14 +302,23 @@ pclose = function ()
     _plend1 ();
     _plsstrm (strtod (members (WIN)[1]));
     P = strtod (members (WIN)[1]);
-  else
+    return P;
+
+  else if (size (WIN) == 1) {
+
     if (exist (WIN.[P])) 
     { 
       clear (WIN.[P]); 
       clear (P);
     }
     _plend1 ();
-  }
+    return 1;
+
+  else if (size (WIN) == 0) {
+
+    return 0;
+
+  } } }
 };
 
 ##############################################################################
@@ -504,20 +550,28 @@ subplot = function ( sub )
 {
   check_plot_object ();
   
-  if (!exist (sub)) 
-  { 
-    sub = 0;
-    #WIN.[P].subplot = WIN.[P].subplot + 1;
+  if (!exist (sub))
+  {
     subplot_f = 0;
     _pladv (0);
   else
-    if (sub > WIN.[P].nplot) 
-    { 
+    if (sub > WIN.[P].nplot)
+    {
       error ("Current window does not have this many subplots");
     }
-    WIN.[P].subplot = sub - 1;
-    subplot_f = 1;
-    _pladv (sub);
+    if (sub > 0)
+    {
+      WIN.[P].subplot = sub - 1;
+      subplot_f = 1;
+      _pladv (sub);
+    else
+      if (sub == 0)
+      {
+        # Do not advance, stay at current subplot
+        WIN.[P].subplot = WIN.[P].subplot - 1;
+        subplot_f = 1;
+      }
+    }
   }
 };
 
@@ -824,7 +878,7 @@ plot3 = function ( L31, L32, L33 )
   _plfont (WIN.[P].font[p]);
   _plwid (WIN.[P].width[p]);
   
-  basex = 2; basey = 2; height = 4;
+  # basex = 2; basey = 2; height = 4;
   xmin2d = -2.0; xmax2d = 2.0;
   ymin2d = -3.0; ymax2d = 5.0;
   
@@ -950,7 +1004,7 @@ plmesh = function ( L31, L32, L33 )
   _plfont (WIN.[P].font[p]);
   _plwid (WIN.[P].width[p]);
   
-  basex = 2; basey = 2; height = 4;
+  # basex = 2; basey = 2; height = 4;
   xmin2d = -2.0; xmax2d = 2.0;
   ymin2d = -3.0; ymax2d = 5.0;
   
@@ -1206,11 +1260,11 @@ pl3d = function ( X, Y, Z, BR )
   if (find_char (WIN.[P].grid3z[p], "l"))
   { Z = log10 (real (Z)); else Z = real (Z); }
     
-  basex = 2; basey = 2; height = 4;
+  # basex = 2; basey = 2; height = 4;
   xmin2d = -2.0; xmax2d = 2.0;
   ymin2d = -3.0; ymax2d = 5.0;
   
-  _plvpas (0.15, 0.85, 0.15, 0.85, WIN.[P].aspect[p]);
+  _plvasp (WIN.[P].aspect[p]);
   _plwind (xmin2d, xmax2d, ymin2d, ymax2d);
   _plw3d (basex, basey, height, xmin, xmax, ymin, ymax, ...
           zmin, zmax, WIN.[P].alt[p], WIN.[P].az[p]);
@@ -1248,7 +1302,7 @@ plerry = function (x, y, y_low, y_high)
   check_plot_object ();
   
   p = mod (WIN.[P].subplot, WIN.[P].nplot) + 1;	# The current index
-  WIN.[P].desc.[p] = inf(); // use own legend
+  WIN.[P].desc.[p] = 1j;    // use own legend
   
   if (x.nr != y.nr || x.nr != y_low.nr || x.nr != y_high.nr) 
   {
@@ -1323,8 +1377,8 @@ plhist = function ( M , nbin )
   # Check computed scale limits against user's
   #
   
-  if (!isinf (WIN.[P].ymin[p])) { ymin = WIN.[P].ymin[p]; }
-  if (!isinf (WIN.[P].ymax[p])) { ymax = WIN.[P].ymax[p]; }
+  if (WIN.[P].ymin[p] != 1j) { ymin = WIN.[P].ymin[p]; }
+  if (WIN.[P].ymax[p] != 1j) { ymax = WIN.[P].ymax[p]; }
 
   _plgra ();
   _plcol (1);
@@ -1360,7 +1414,7 @@ plhist = function ( M , nbin )
     _plcol (WIN.[P].color[p;k]);
     _plhist (np, real(M[;i]), ymin, ymax, nbin, 1);
     
-    if (!any (any (isinf (WIN.[P].desc.[p]))))
+    if (!any (any (WIN.[P].desc.[p] == 1j)))
     {
       # Use the default if necessary
       if (WIN.[P].desc.[p][1] == "default") 
@@ -1479,39 +1533,39 @@ plimits = function ( xmin, xmax, ymin, ymax, zmin, zmax )
   {
     WIN.[P].xmin[i] = xmin;
   else
-    WIN.[P].xmin[i] = inf ();
+    WIN.[P].xmin[i] = 1j;
   }
   if (exist (xmax)) 
   { 
     WIN.[P].xmax[i] = xmax;
   else
-    WIN.[P].xmax[i] = inf ();
+    WIN.[P].xmax[i] = 1j;
   }
 
   if (exist (ymin)) 
   {
     WIN.[P].ymin[i] = ymin;
   else
-    WIN.[P].ymin[i] = inf ();
+    WIN.[P].ymin[i] = 1j;
   }
   if (exist (ymax)) 
   {
     WIN.[P].ymax[i] = ymax;
   else
-    WIN.[P].ymax[i] = inf ();
+    WIN.[P].ymax[i] = 1j;
   }
 
   if (exist (zmin)) 
   {
     WIN.[P].zmin[i] = zmin;
   else
-    WIN.[P].zmin[i] = inf ();
+    WIN.[P].zmin[i] = 1j;
   }
   if (exist (zmax)) 
   {
     WIN.[P].zmax[i] = zmax;
   else
-    WIN.[P].zmax[i] = inf ();
+    WIN.[P].zmax[i] = 1j;
   }
 };
 
@@ -1650,8 +1704,8 @@ x_scales = function ( M, p, xmin, xmax )
   # Check computed scale limits against user's
   #
   
-  if (!isinf (WIN.[P].xmin[p])) { xmin = WIN.[P].xmin[p]; }
-  if (!isinf (WIN.[P].xmax[p])) { xmax = WIN.[P].xmax[p]; }
+  if (WIN.[P].xmin[p] != 1j) { xmin = WIN.[P].xmin[p]; }
+  if (WIN.[P].xmax[p] != 1j) { xmax = WIN.[P].xmax[p]; }
 
   #
   # Check for potential errors
@@ -1697,8 +1751,8 @@ y_scales = function ( M, p, xmin, xmax )
   # Check computed scale limits against user's
   #
   
-  if (!isinf (WIN.[P].ymin[p])) { xmin = WIN.[P].ymin[p]; }
-  if (!isinf (WIN.[P].ymax[p])) { xmax = WIN.[P].ymax[p]; }
+  if (WIN.[P].ymin[p] != 1j) { xmin = WIN.[P].ymin[p]; }
+  if (WIN.[P].ymax[p] != 1j) { xmax = WIN.[P].ymax[p]; }
 
   #
   # Check for potential errors
@@ -1744,8 +1798,8 @@ z_scales = function ( M, p, xmin, xmax )
   # Check computed scale limits against user's
   #
   
-  if (!isinf (WIN.[P].zmin[p])) { xmin = WIN.[P].zmin[p]; }
-  if (!isinf (WIN.[P].zmax[p])) { xmax = WIN.[P].zmax[p]; }
+  if (WIN.[P].zmin[p] != 1j) { xmin = WIN.[P].zmin[p]; }
+  if (WIN.[P].zmax[p] != 1j) { xmax = WIN.[P].zmax[p]; }
 
   #
   # Check for potential errors
@@ -1805,10 +1859,10 @@ xy_scales = function ( M, p, xmin, xmax, ymin, ymax )
   # Check computed scale limits against user's
   #
   
-  if (!isinf (WIN.[P].xmin[p])) { xmin = WIN.[P].xmin[p]; }
-  if (!isinf (WIN.[P].xmax[p])) { xmax = WIN.[P].xmax[p]; }
-  if (!isinf (WIN.[P].ymin[p])) { ymin = WIN.[P].ymin[p]; }
-  if (!isinf (WIN.[P].ymax[p])) { ymax = WIN.[P].ymax[p]; }
+  if (WIN.[P].xmin[p] != 1j) { xmin = WIN.[P].xmin[p]; }
+  if (WIN.[P].xmax[p] != 1j) { xmax = WIN.[P].xmax[p]; }
+  if (WIN.[P].ymin[p] != 1j) { ymin = WIN.[P].ymin[p]; }
+  if (WIN.[P].ymax[p] != 1j) { ymax = WIN.[P].ymax[p]; }
   
   #
   # Check for potential errors
@@ -1884,12 +1938,12 @@ XYZ_scales = function ( X, Y, Z, p, xmin, xmax, ymin, ymax, zmin, zmax )
   # Check computed scale limits against user's
   #
   
-  if (!isinf (WIN.[P].xmin[p])) { xmin = WIN.[P].xmin[p]; }
-  if (!isinf (WIN.[P].xmax[p])) { xmax = WIN.[P].xmax[p]; }
-  if (!isinf (WIN.[P].ymin[p])) { ymin = WIN.[P].ymin[p]; }
-  if (!isinf (WIN.[P].ymax[p])) { ymax = WIN.[P].ymax[p]; }
-  if (!isinf (WIN.[P].zmin[p])) { zmin = WIN.[P].zmin[p]; }
-  if (!isinf (WIN.[P].zmax[p])) { zmax = WIN.[P].zmax[p]; }
+  if (WIN.[P].xmin[p] != 1j) { xmin = WIN.[P].xmin[p]; }
+  if (WIN.[P].xmax[p] != 1j) { xmax = WIN.[P].xmax[p]; }
+  if (WIN.[P].ymin[p] != 1j) { ymin = WIN.[P].ymin[p]; }
+  if (WIN.[P].ymax[p] != 1j) { ymax = WIN.[P].ymax[p]; }
+  if (WIN.[P].zmin[p] != 1j) { zmin = WIN.[P].zmin[p]; }
+  if (WIN.[P].zmax[p] != 1j) { zmax = WIN.[P].zmax[p]; }
   
   #
   # Check for potential errors
@@ -2084,7 +2138,7 @@ plot_matrix = function ( M, key, p, K, xmin, xmax, ymin, ymax, v )
     # Now do the legend 
     #
     
-    if (!any (any (isinf (WIN.[P].desc.[p]))))
+    if (!any (any (WIN.[P].desc.[p] == 1j)))
     {
       # Use the default if necessary
       if (WIN.[P].desc.[p][1] == "default") 
@@ -2172,7 +2226,7 @@ plot_matrix = function ( M, key, p, K, xmin, xmax, ymin, ymax, v )
       # Now do the legend 
       #
       
-      if (!any (any (isinf (WIN.[P].desc.[p]))))
+      if (!any (any (WIN.[P].desc.[p] == 1j)))
       {
 	# Use the default if necessary
 	if (WIN.[P].desc.[p][1] == "default") 
@@ -2325,8 +2379,8 @@ plhistx = function ( M , nbin )
   # Check computed scale limits against user's
   #
   
-  if (!isinf (WIN.[P].ymin[p])) { ymin = WIN.[P].ymin[p]; }
-  if (!isinf (WIN.[P].ymax[p])) { ymax = WIN.[P].ymax[p]; }
+  if (WIN.[P].ymin[p] != 1j) { ymin = WIN.[P].ymin[p]; }
+  if (WIN.[P].ymax[p] != 1j) { ymax = WIN.[P].ymax[p]; }
   
   _plgra ();
   _plcol (15);
@@ -2392,7 +2446,7 @@ plhistx = function ( M , nbin )
     // write legend around upper-right corner.
     // it is better to have user to choose location for legend.
     
-    if (!any (any (isinf (WIN.[P].desc.[p]))))
+    if (!any (any (WIN.[P].desc.[p] == 1j)))
     {
       # Use the default if necessary
       if (WIN.[P].desc.[p][1] == "default") 
@@ -2460,7 +2514,7 @@ plegend = function ( LEGEND )
   
   if (!exist (LEGEND)) 
   {
-    WIN.[P].desc.[p] = inf();
+    WIN.[P].desc.[p] = 1j;
     return P;
   }
   
@@ -2471,3 +2525,10 @@ plegend = function ( LEGEND )
   
   return P;
 };
+
+set3d = function (bx, by, h)
+{
+  if (!exist (bx)) { basex = 2; else basex = bx; }
+  if (!exist (by)) { basey = 2; else basey = by; }
+  if (!exist (h)) { height = 4; else height = h; }
+};
