@@ -2,7 +2,7 @@
 /* Matrix class operations for the RLaB machine. */
 
 /*  This file is a part of RLaB ("Our"-LaB)
-   Copyright (C) 1992, 1994  Ian R. Searle
+   Copyright (C) 1992, 1994, 1995  Ian R. Searle
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -669,7 +669,17 @@ matrix_assign (i_flag, var, i1, i2, a)
     matrix_gt_zero (mrow);
     matrix_gt_zero (mcol);
 
-    matrix_Assign (m, mrow, mcol, e_data (a.u.ent));
+    if (m != (Matrix *) e_data (a.u.ent))
+    {
+      matrix_Assign (m, mrow, mcol, e_data (a.u.ent));
+    }
+    else
+    {
+      Matrix *mtmp;
+      mtmp = matrix_Copy (e_data (a.u.ent));
+      matrix_Assign (m, mrow, mcol, mtmp);
+      matrix_Destroy (mtmp);
+    }
 
     remove_tmp_destroy (i1.u.ent);
     remove_tmp_destroy (i2.u.ent);
@@ -688,7 +698,17 @@ matrix_assign (i_flag, var, i1, i2, a)
 
     matrix_gt_zero (mrow);
 
-    matrix_AssignCol (m, mrow, e_data (a.u.ent));
+    if (m != (Matrix *) e_data (a.u.ent))
+    {
+      matrix_AssignCol (m, mrow, e_data (a.u.ent));
+    }
+    else
+    {
+      Matrix *mtmp;
+      mtmp = matrix_Copy (e_data (a.u.ent));
+      matrix_AssignCol (m, mrow, mtmp);
+      matrix_Destroy (mtmp);
+    }
 
     remove_tmp_destroy (i1.u.ent);
     remove_tmp_destroy (a.u.ent);
@@ -706,7 +726,17 @@ matrix_assign (i_flag, var, i1, i2, a)
 
     matrix_gt_zero (mcol);
 
-    matrix_AssignRow (m, mcol, e_data (a.u.ent));
+    if (m != (Matrix *) e_data (a.u.ent))
+    {
+      matrix_AssignRow (m, mcol, e_data (a.u.ent));
+    }
+    else
+    {
+      Matrix *mtmp;
+      mtmp = matrix_Copy (e_data (a.u.ent));
+      matrix_AssignRow (m, mcol, mtmp);
+      matrix_Destroy (mtmp);
+    }
 
     remove_tmp_destroy (i2.u.ent);
     remove_tmp_destroy (a.u.ent);
@@ -830,7 +860,17 @@ matrix_vector_assign (lhs, ind, rhs)
     error_1 (e_name (lhs.u.ent), "col indices must be REAL");
 
   matrix_gt_zero (mi);
-  matrix_AssignVector (m, mi, mrhs);
+  if (m != mrhs)
+  {
+    matrix_AssignVector (m, mi, mrhs);
+  }
+  else
+  {
+    Matrix *mtmp;
+    mtmp = matrix_Copy (mrhs);
+    matrix_AssignVector (m, mi, mtmp);
+    matrix_Destroy (mtmp);
+  }
 
   remove_tmp_destroy (ind.u.ent);
   remove_tmp_destroy (rhs.u.ent);
@@ -1014,6 +1054,10 @@ matrix_reshape_col (d)
   {
     new = d;
   }
+  else if (MNR (m) == 0 && MNC (m) == 0)
+  {
+    new = d;
+  }
   else
   {
     /* Now do the reshape */
