@@ -133,7 +133,7 @@ Close (return_ptr, n_args, d_arg)
   ListNode *F;
 
   if (n_args != 1)
-    error_1 ("close: requires an arg", 0);
+    error_1 ("close: 1 argument allowed", 0);
 
   F = bltin_get_string ("close", d_arg, 1);
 
