@@ -36,7 +36,7 @@
 #include <sys/types.h>
 #include <stdio.h>
 #include <errno.h>
-#ifdef HAVE_DIRENT
+#ifdef HAVE_DIRENT_H
 #include <dirent.h>
 #else
 #include <sys/dir.h>
@@ -90,6 +90,7 @@ void print_rfiles _PROTO ((char *dirname, FILE * fptr));
 int find_open _PROTO ((char *filenm, char *dirname));
 int rfile_dir _PROTO ((char *dir));
 Path *path_split _PROTO ((char *path));
+Path *name_split _PROTO ((char *path));
 void path_Destroy _PROTO ((Path * path));
 
 static void swap _PROTO ((char *v[], int i, int j));
@@ -208,39 +209,127 @@ rfile ()
 #endif /* THINK_C */
 
 /*
- * Interpret an rfile (load it).
+ * Add the `.r' extension to a name if it is not
+ * there. Do nothing if it is not.
+ */
+
+char *
+rfile_ext (name)
+     char *name;
+{
+  char *nm, *np;
+  int len = strlen (name);
+
+  /* On the off chance someone hands a zero length name. */
+  if (len == 0)
+    return (0);
+
+  /* Check for the `.r' extension. */
+  np = name;
+  if (np[len-1] == 'r' && np[len-2] == '.')
+  {
+    /* name already has extension, do nothing. */
+    return (nm = cpstr (name));
+  }
+  else
+  {
+    /* Add the `.r' to name. */
+    nm = MALLOC (sizeof (char) * len + 3);
+    strcpy (nm, name);
+    strcat (nm, ".r");
+    nm[len+2] = '\0';
+  }
+  return (nm);
+}
+
+/*
+ * Interpret a rfile (load it).
  * Argument `name' is destroyed by rfile_load.
+ * name can be a single name-string or white-space separated
+ * string of names. Each string can have the postfix .r or not.
  */
 
 void
 rfile_load (name)
      char *name;
 {
-  char tmp[100];
-  int i;
-  Path *path;
+  char *nm;
+  int fval, i, j;
+  Path *names, *path;
+  
+  /* Initialize */
+  fval = 0;
 
-  /* Append the `.r' to the name */
-  /* FIX */
-  strcpy (tmp, name);
-  strcat (tmp, ".r");
+  /* Split up name if necessary. */
+  names = name_split (name);
+  if (names == 0)
+  {
+    fprintf (stderr, "WARNING, invalid name supplied to rfile\n");
+    return;
+  }
 
   /* Re-split the PATH, in case it has changed */
   if ((l_search_path = get_rlab_search_path ()) == 0)
     l_search_path = search_path;
   path = path_split (l_search_path);
 
+  /* Load each file in the command. */
+  for (j = 0; j < names->ndir; j++)
+  {
+    nm = rfile_ext (names->dira[j]);
+    if (nm == 0) 
+      continue;
+
+    /* Check each directory in the path. */
+    for (i = 0; i < path->ndir; i++)
+    {
+      if ((fval = find_open (nm, path->dira[i])) == 1)
+      {
+	break; /* We found/loaded it. */
+      }
+    }
+
+    if (fval == 0)
+      fprintf (stderr, "WARNING, could not find/open: %s\n", nm);
+    FREE (nm);
+  }
+
+  FREE (name);
+  path_Destroy (names);
+  path_Destroy (path);
+}
+
+void
+rfile_load_one (name)
+     char *name;
+{
+  char *nm;
+  int fval, i;
+  Path *path;
+  
+  /* Initialize */
+  fval = 0;
+
+  /* Re-split the PATH, in case it has changed */
+  if ((l_search_path = get_rlab_search_path ()) == 0)
+    l_search_path = search_path;
+  path = path_split (l_search_path);
+
+  nm = rfile_ext (name);
+
+  /* Check each directory in the path. */
   for (i = 0; i < path->ndir; i++)
   {
-    if ((find_open (tmp, path->dira[i])) != 0)
+    if ((fval = find_open (nm, path->dira[i])) == 1)
     {
-      FREE (name);
-      path_Destroy (path);
-      return;
+      break; /* We found/loaded it. */
     }
   }
-  fprintf (stderr, "ERROR, could not find/open: %s\n", tmp);
-  FREE (name);
+  
+  if (fval == 0)
+    fprintf (stderr, "WARNING, could not find/open: %s\n", nm);
+
+  FREE (nm);
   path_Destroy (path);
 }
 
@@ -290,6 +379,57 @@ path_split (path)
   return (new);
 }
 
+/*
+ * Split a whitespace separated namelist into distinct
+ * character strings.
+ * EX: `poly roots bode'
+ * Return a pointer to an array of character pointers.
+ */
+
+Path *
+name_split (name)
+     char *name;
+{
+  int i, n, ndir, start;
+  char **dira;
+  Path *new;
+
+  if (name == 0 && strlen (name) == 0)
+    return (0);
+
+  new = (Path *) MALLOC (sizeof (Path));
+  ndir = 1;
+  i = 0;
+  /* Count the number of directories */
+  while (name[i] != '\0')
+  {
+    if (name[i] == ' ')
+      ndir++;
+    i++;
+  }
+  /* Get space for char pts */
+  dira = (char **) MALLOC (ndir * sizeof (char *));
+
+  /* Now get directory strings */
+  start = 0;
+  for (i = 0; i < ndir; i++)
+  {
+    /* Count length of string */
+    n = 0;
+    while (name[start + n] != ' ' && name[start + n] != '\0')
+      n++;
+
+    /* Now copy string into dira */
+    dira[i] = (char *) MALLOC ((n + 1) * sizeof (char));
+    strncpy (dira[i], &name[start], n);
+    dira[i][n] = '\0';
+    start += (n + 1);
+  }
+  new->ndir = ndir;
+  new->dira = dira;
+  return (new);
+}
+
 #ifdef THINK_C
 static int
 more (FILE * fptr, char *msg)
@@ -379,7 +519,7 @@ print_rfiles (dirname, fptr)
   char **names;
   int i, length, ncol, nfile;
   DIR *dirp;
-#ifdef HAVE_DIRENT
+#ifdef HAVE_DIRENT_H
   struct dirent *direntp;
 #else
   struct direct *direntp;
@@ -492,7 +632,7 @@ find_open (filenm, dirname)
   /* FIX */
   char tmp[100];
   DIR *dirp;
-#ifdef HAVE_DIRENT
+#ifdef HAVE_DIRENT_H
   struct dirent *direntp;
 #else
   struct direct *direntp;
@@ -634,7 +774,7 @@ help ()
   char **names;
   int i, length, ncol, nfile;
   DIR *dirp;
-#ifdef HAVE_DIRENT
+#ifdef HAVE_DIRENT_H
   struct dirent *direntp;
 #else
   struct direct *direntp;
@@ -743,7 +883,7 @@ help ()
   char **names;
   int i, length, ncol, nfile;
   DIR *dirp;
-#ifdef HAVE_DIRENT
+#ifdef HAVE_DIRENT_H
   struct dirent *direntp;
 #else
   struct direct *direntp;
@@ -857,7 +997,7 @@ help_name (name)
   char *tmp;
   int i, length;
   DIR *dirp;
-#ifdef HAVE_DIRENT
+#ifdef HAVE_DIRENT_H
   struct dirent *direntp;
 #else
   struct direct *direntp;
@@ -1003,7 +1143,7 @@ find_full_filename (rfilenm, dirname)
   char tmp[100], *fullname;
   int length;
   DIR *dirp;
-#ifdef HAVE_DIRENT
+#ifdef HAVE_DIRENT_H
   struct dirent *direntp;
 #else
   struct direct *direntp;
@@ -1058,3 +1198,79 @@ get_rlab_search_path ()
 
   return (0);
 }
+
+/*
+ * Make sure there is no .r
+ */
+
+char *
+no_ext (name)
+     char *name;
+{
+  char *nm, *np;
+  int len = strlen (name);
+
+  /* On the off chance someone hands a zero length name. */
+  if (len == 0)
+    return (0);
+
+  /* Check for the `.r' extension. */
+  np = name;
+  if (np[len-1] == 'r' && np[len-2] == '.')
+  {
+    /* name already has extension, strip it. */
+    nm = MALLOC (sizeof (char) * len - 1);
+    strncpy (nm, name, len - 2);
+    return (nm);
+  }
+  else
+  {
+    /* Do nothing. */
+    return (nm = cpstr (name));
+  }
+  return (nm);
+}
+
+void
+require (name)
+     char *name;
+{
+  char *func_name;
+  int i;
+  ListNode *ent;
+  Path *names;
+
+  names = name_split (name);
+  if (names == 0)
+  {
+    fprintf (stderr, "WARNING, invalid name supplied to require\n");
+    return;
+  }
+
+  for (i = 0; i < names->ndir; i++)
+  {
+    /* Get the function name. */
+    func_name = no_ext (names->dira[i]);
+
+    /* Now look for it in the symbol table. */
+    if ((ent = lookup (0, func_name)) != 0)
+    {
+      /* Now we must check to see if there is something there */
+      if (e_type (ent) == UNDEF)
+      {
+	/* Load the file. */
+	rfile_load_one (names->dira[i]);
+      }
+      /* Else, do nothing. */
+    }
+    else
+    {
+      /* Load the file. */
+      rfile_load_one (names->dira[i]);
+    }
+    FREE (func_name);
+  }
+
+  FREE (name);
+  path_Destroy (names);
+}
