@@ -342,6 +342,9 @@ warning_3 (s1, s2, s3, t)
 /* **************************************************************
  * Signal catching functions.
  * ************************************************************** */
+
+#include "fpe.h"
+
 void
 fpecatch (tmp)
      int tmp;
@@ -351,6 +354,15 @@ fpecatch (tmp)
   signal (SIGFPE, SIG_ACK);
 #endif
 
+  setup_fpe_handling ();
+
+  /*
+   * This may be over-cautious (paranoid), but it is
+   * possible that we could get another one of these
+   * on our way back to the prompt, so lets ignore 
+   * any more that come along.
+   */
+
   signal (SIGFPE, SIG_IGN);
   error_1 ("floating point exception", 0);
 }
@@ -373,6 +385,7 @@ pipecatch (tmp)
  * Copy a string to a new char ptr, Return the ptr to the newly
  * created string.
  * ************************************************************** */
+
 char *
 cpstr (string)
      char *string;
@@ -387,10 +400,43 @@ cpstr (string)
   return (0);
 }
 
+/* **************************************************************
+ * Copy  a string to a new char ptr, Return the ptr to the newly
+ * created string. Copy at most N chars. Always terminate with a 
+ * null character. 
+ * ************************************************************** */
+
+char *
+cpnstr (string, n)
+     char *string;
+     int n;
+{
+  char *new_string;
+  if (string != 0)
+  {
+    int length = strlen (string);
+    if (length >= n)
+    {
+      new_string = (char *) MALLOC ((n+1) * sizeof (char));
+      strncpy (new_string, string, n);
+      new_string[n] = '\0';
+    }
+    else
+    {
+      new_string = (char *) MALLOC ((length + 1) * sizeof (char));
+      strncpy (new_string, string, length);
+      new_string[length] = '\0';
+    }
+    return (new_string);
+  }
+  return (0);
+}
+
 /* **************************************************************
  * Copy a string to a new char *, strip the surrounding "".
  * This function is primarily used by the RLaB scanner.
  * ************************************************************** */
+
 char *
 cpstr_strip (string)
      char *string;
@@ -411,6 +457,29 @@ cpstr_strip (string)
   return (0);
 }
 
+/* **************************************************************
+ * Append a string to another, separated with whitespace.
+ * Return the new result.
+ * ************************************************************** */
+
+char *
+strappend (s1, s2)
+     char *s1, *s2;
+{
+  int l1, l2;
+  char *s;
+
+  l1 = strlen (s1);
+  l2 = strlen (s2);
+
+  s = CALLOC (l1 + l2 + 2, sizeof (char));
+  strcpy (s, s1);
+  strcat (s, " ");  /* Separate with whitespace. */
+  strcat (s, s2);
+
+  return (s);
+}
+
 /* **************************************************************
  * Get a double value from a Datum.
  * ************************************************************** */
@@ -1076,7 +1145,7 @@ convert_to_rhs_matrix (d)
  * Each variant gets certain types of nodes from the TREE.
  * ************************************************************** */
 
-#define N_WORD     5		/* Number of columns to print for what(), and who(). */
+#define N_WORD     5	/* Number of columns to print for what(), and who(). */
 static int count = 0;
 
 /*
@@ -1085,20 +1154,20 @@ static int count = 0;
  * the return'ed object is the caller's.
  */
 
-static char **
-btree_get_node_names (node, names)
+char **
+get_btree_node_names (node, names)
      ListNode *node;
      char **names;
 {
   if (node != 0)
   {
-    names = btree_get_node_names (node->prev, names);
+    names = get_btree_node_names (node->prev, names);
     if (listNode_GetType (node) != UNDEF)
     {
       *(names++) = cpstr (listNode_GetKey (node));
       count++;
     }
-    names = btree_get_node_names (node->next, names);
+    names = get_btree_node_names (node->next, names);
   }
   return (names);
 }
@@ -1242,7 +1311,7 @@ btree_members (root)
   count = 0;
   names = (char **) MALLOC ((root->numNodes) * sizeof (char *));
 
-  btree_get_node_names (root->root_node, names);
+  get_btree_node_names (root->root_node, names);
 
   m = matrix_CreateS (1, count);
 
